'use strict';

exports.__esModule = true;
exports.default = createStackFilter;

var _path = require('path');

var _lodash = require('lodash');

const BABEL = require.resolve('babel-core');
const BABEL_MODULES_DIR = BABEL.replace(new RegExp(`^(.*${(0, _lodash.escapeRegExp)(_path.sep)}node_modules${(0, _lodash.escapeRegExp)(_path.sep)})(.*)`), '$1');

const BABEL_RELATED = BABEL_MODULES_DIR + 'babel-';
const BABYLON = BABEL_MODULES_DIR + 'babylon' + _path.sep;
const CORE_JS = BABEL_MODULES_DIR + 'core-js' + _path.sep;
const REGENERATOR_RUNTIME = BABEL_MODULES_DIR + 'regenerator-runtime' + _path.sep;

const TESTCAFE_LIB = (0, _path.join)(__dirname, '../');
const TESTCAFE_BIN = (0, _path.join)(__dirname, '../../bin');
const TESTCAFE_HAMMERHEAD = `${_path.sep}testcafe-hammerhead${_path.sep}`;

const SOURCE_MAP_SUPPORT = `${_path.sep}source-map-support${_path.sep}`;

const INTERNAL = 'internal/';

function createStackFilter(limit) {
    var passedFramesCount = 0;

    return function stackFilter(frame) {
        if (passedFramesCount >= limit) return false;

        var filename = frame.getFileName();

        // NOTE: filter out the internals of node, Babel and TestCafe
        var pass = filename && filename.indexOf(_path.sep) > -1 && filename.indexOf(INTERNAL) !== 0 && filename.indexOf(TESTCAFE_LIB) !== 0 && filename.indexOf(TESTCAFE_BIN) !== 0 && filename.indexOf(TESTCAFE_HAMMERHEAD) < 0 && filename.indexOf(BABEL_RELATED) !== 0 && filename.indexOf(BABYLON) !== 0 && filename.indexOf(CORE_JS) !== 0 && filename.indexOf(REGENERATOR_RUNTIME) !== 0 && filename.indexOf(SOURCE_MAP_SUPPORT) < 0;

        if (pass) passedFramesCount++;

        return pass;
    };
}
module.exports = exports['default'];