'use strict';

exports.__esModule = true;

var _pinkie = require('pinkie');

var _pinkie2 = _interopRequireDefault(_pinkie);

var _lodash = require('lodash');

var _testRunTracker = require('../api/test-run-tracker');

var _testRunTracker2 = _interopRequireDefault(_testRunTracker);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

class ReExecutablePromise extends _pinkie2.default {
    constructor(executorFn) {
        super(_lodash.noop);

        this._fn = executorFn;
        this._taskPromise = null;
    }

    _ensureExecuting() {
        if (!this._taskPromise) this._taskPromise = new _pinkie2.default(this._fn);
    }

    _reExecute() {
        this._taskPromise = null;

        return this;
    }

    then(onFulfilled, onRejected) {
        this._ensureExecuting();

        return this._taskPromise.then(onFulfilled, onRejected);
    }

    catch(onRejected) {
        this._ensureExecuting();

        return this._taskPromise.catch(onRejected);
    }

    static fromFn(asyncExecutorFn) {
        var testRunId = _testRunTracker2.default.getContextTestRunId();

        if (testRunId) asyncExecutorFn = _testRunTracker2.default.addTrackingMarkerToFunction(testRunId, asyncExecutorFn);

        return new ReExecutablePromise(resolve => resolve(asyncExecutorFn()));
    }
}
exports.default = ReExecutablePromise;
module.exports = exports['default'];